/**
 * Copyright (c) 2006, Van
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the VAN nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
package van.harvest;

import com.ca.harvest.jhsdk.*;
import com.ca.harvest.jhsdk.hutils.JCaContainer;
import com.ca.harvest.jhsdk.hutils.JCaHarvestException;
import com.intellij.localVcs.fileView.LvcsFileViewEnvironment;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.localVcs.LocalVcs;
import com.intellij.openapi.localVcs.LocalVcsServices;
import com.intellij.openapi.options.Configurable;
import com.intellij.openapi.vcs.*;
import com.intellij.openapi.vcs.checkin.CheckinEnvironment;
import com.intellij.openapi.vcs.fileView.FileViewEnvironment;
import com.intellij.openapi.components.ProjectComponent;
import van.harvest.model.HarvestManager;

import java.io.*;
import java.util.ArrayList;
import java.util.Map;


public class HarvestVcs extends AbstractVcs implements ProjectComponent,StandardOperationsProvider {
    private static final Logger LOG = Logger.getInstance("#van.harvest.HarvestVcs");

    private HarvestConfiguration myConfiguration;

    public static final String VIEW_NOT_FOUND = "View {0} not found";
    public static final String PROJECT_NOT_FOUND = "Project {0} not found";
    public static final String FILE_NOT_FOUND_IN_STARTEAM = "File {0} not found in Harvest repository";
    public static final String FOLDER_NOT_FOUND_IN_STARTEAM = "Folder {0} was not found in Harvest repository";
    public static final String FOLDER_NOT_FOUND_ON_DISK = "Folder {0} was not found on disk";
    public static final String FILE_NOT_FOUND_ON_DISK = "File {0} not found on disk";

    private static final char SEP = java.io.File.separatorChar;
    private final LvcsFileViewEnvironment myFileViewEnvironment;
    private CheckinEnvironment myCheckinEnvironment;

    public HarvestVcs(com.intellij.openapi.project.Project project,
                      LocalVcsServices localVcsServices,
                      HarvestConfiguration harvestConfiguration) {
        super(project);
        myFileViewEnvironment = new LvcsFileViewEnvironment(myProject);
        myConfiguration = harvestConfiguration;
        myCheckinEnvironment = localVcsServices.createCheckinEnvironment(this);
    }

    public String getName() {
        return "Harvest";
    }

    public String getDisplayName() {
        return "Harvest";
    }

    public void projectOpened() {
       loadSettings();
    }

    public void projectClosed() {
    }

    public void disposeComponent() {
        myCheckinEnvironment = null;
    }

    public void initComponent() {
    }

    public void start() throws VcsException {
        super.start();
        try {
            LOG.debug("enter: start()");

            JCaHarvest harvest = HarvestManager.getInstance().getHarvest(myProject);
            if (harvest == null) return;

            LOG.debug("exit: start()");
        } catch (Exception e) {
            LOG.debug(e);
            throw new VcsException(e);
        }
    }

    public void shutdown() throws VcsException {
        LOG.debug("enter: shutdown()");

        disconnect();

        LOG.debug("exit: shutdown()");
        super.shutdown();
    }

    private void disconnect() {
        HarvestManager.getInstance().logout(myProject);
    }

    public void checkinFile(String path, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: checkinFile(file='" + path + "')");
        }
        internalCheckin(path, JCaConst.HAR_UPDATE_AND_RELEASE, parameters);

    }

    public void checkinKeepFile(String path, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: checkinKeepFile(file='" + path + "')");
        }
        internalCheckin(path, JCaConst.HAR_UPDATE_AND_KEEP, parameters);
    }

    private void internalCheckin(String path, int iMode, Object parameters) throws VcsException {

        String fileName = path.substring(path.lastIndexOf(SEP) + 1);
        String clientPath = path.substring(0, path.lastIndexOf(SEP));
        MapItem configItem = myConfiguration.findItem(path);
        String viewPath = getViewPath(configItem, clientPath);
        JCaContext context = HarvestManager.getInstance().getContext(myProject);
        context.setProject(configItem.PROJECT);
        context.setState(configItem.STATE);
        context.setPackage(configItem.PACKAGE);
        String comment = (String) parameters;
        context.setCheckin("Check In Changes");
        JCaCheckin checkin = context.getCheckin();
        if (comment != null)
            checkin.setDescription(comment);
        checkin.setCheckinMode(iMode);
        checkin.setClientDir(clientPath);
        //String sViewPath = context.getViewPath();
        checkin.setViewPath(viewPath);
        checkin.setDeleteAfterCI(false);
        checkin.setPathOption(JCaConst.CI_OPTION_PRESERVE_AND_CREATE_PATH);
        checkin.setItemOption(JCaConst.CI_FILTER_EXISTING_ITEMS_ONLY);
        String fileNames[] = new String[]{fileName};

        if (fileNames.length > 0) {
            checkin.setFileList(fileNames);
            int iResult = checkin.execute();
            if (iResult != 0)
                throw new VcsException("checkin fail." + context.getCheckinName() + path);
        }
    }

    public boolean checkoutFile(String path, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: checkoutFile(file='" + path + "')");
        }
        return internalCheckout(path, JCaConst.CO_MODE_UPDATE, parameters);
    }

    public boolean checkoutFileForBrowse(String path, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: checkoutFileForBrowse(file='" + path + "')");
        }
        return internalCheckout(path, JCaConst.CO_MODE_BROWSE, parameters);
    }

    private boolean internalCheckout(String path, int oMode, Object parameters) throws VcsException {
        String fileName = path.substring(path.lastIndexOf(SEP) + 1);
        String clientPath = path.substring(0, path.lastIndexOf(SEP));
        MapItem configItem = myConfiguration.findItem(path);
        String viewPath = getViewPath(configItem, clientPath);
        JCaContext context = HarvestManager.getInstance().getContext(myProject);
        context.setProject(configItem.PROJECT);
        context.setState(configItem.STATE);
        context.setPackage(configItem.PACKAGE);

        if (context.getPackageId() == -1)
            throw new VcsException("no Package");
        context.setCheckout("Check Out");
        JCaCheckout checkout = context.getCheckout();
        boolean bResult = true;
        bResult = checkout.setCheckoutMode(oMode);
        if (!bResult) throw new VcsException("set Checkout Mode Fail");
        bResult = checkout.setPathOption(JCaConst.CO_OPTION_PRESERVE_AND_CREATE);
        if (!bResult)
            throw new VcsException("set Path Option Fail");
        bResult = checkout.setClientDir(clientPath);
        if (!bResult)
            throw new VcsException("set Client Path Fail:" + clientPath);
        checkout.setViewPath(viewPath);
        checkout.setReplaceFile(true);

        JCaVersionChooser verChooser = context.getVersionChooser();
        try {
            verChooser.setParentPath(viewPath);
        } catch (JCaHarvestException e) {
            throw new VcsException(e);
        }
        verChooser.setRecursive(false);
        verChooser.setBranchOption(JCaConst.BRANCH_FILTER_TRUNK_ONLY);
        verChooser.setVersionOption(JCaConst.VERSION_FILTER_LATEST_IN_VIEW);
        verChooser.setVersionStatusOption(JCaConst.VERSION_FILTER_NORMAL_VERSION);
        verChooser.setItemName(fileName);

        try {
            verChooser.execute();
            JCaContainer container = verChooser.getVersionList();
            int iTotalItems = container.getKeyElementCount("CA_ATTRKEY_NAME");
            if (iTotalItems > 1) {
                LOG.info("enter: checkoutFile(" + iTotalItems + "files)");
            }
            int iResult = checkout.execute();
            if (iResult != 0) {
                throw new VcsException("checkout fail." + context.getCheckoutName() + path);
            }
        } catch (JCaHarvestException e) {
            throw new VcsException(e);
        }


        return true;
    }

    private String getViewPath(MapItem configItem, String clientPath) {
        if (configItem.CLIENT_PATH.equals(clientPath))
            return configItem.VIEW_PATH;
        else
            return configItem.VIEW_PATH + SEP + clientPath.substring(configItem.CLIENT_PATH.length() + 1);
    }

    public boolean undoCheckoutFile(String path, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: checkoutFile(file='" + path + "')");
        }

        internalCheckin(path, JCaConst.HAR_RELEASE_ONLY, parameters);
        return internalCheckout(path, JCaConst.CO_MODE_BROWSE, parameters);
    }


    public byte[] getFileContent(String path, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: getFileContent(file='" + path + "')");
        }
        ByteArrayOutputStream inputStream = new ByteArrayOutputStream();
        FileInputStream outputStream = checkoutTemporaryFile(path, parameters);
        try {
            byte[] buf = new byte[1024];
            int length;
            while ((length = outputStream.read(buf)) != -1) {
                inputStream.write(buf, 0, length);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                outputStream.close();
                inputStream.close();
            } catch (IOException e) {
                //ignore
            }
        }
        return inputStream.toByteArray();
    }

    private FileInputStream checkoutTemporaryFile(String path, Object parameters) throws VcsException {
        String fileName = path.substring(path.lastIndexOf(SEP) + 1);
        String clientPath = path.substring(0, path.lastIndexOf(SEP));

        MapItem configItem = myConfiguration.findItem(path);
        String viewPath = getViewPath(configItem, clientPath);
        JCaContext context = HarvestManager.getInstance().getContext(myProject);
        context.setProject(configItem.PROJECT);
        context.setState(configItem.STATE);
        context.setPackage(configItem.PACKAGE);
        String tempClientPath = System.getProperty("user.home") + SEP + "Local Settings\\Temp";
        tempClientPath = tempClientPath + SEP + "~Temp";
        File clientPathFile = new File(tempClientPath);
        clientPathFile.mkdir();
        clientPathFile.deleteOnExit();
        JCaVersionChooser verChooser = context.getVersionChooser();
        try {
            verChooser.clear();
            verChooser.setParentPath(viewPath);
            verChooser.setRecursive(false);
            verChooser.setBranchOption(JCaConst.BRANCH_FILTER_TRUNK_AND_BRANCH);
            verChooser.setVersionOption(JCaConst.VERSION_FILTER_LATEST_IN_VIEW);
            verChooser.setVersionStatusOption(JCaConst.VERSION_FILTER_NORMAL_VERSION);
            verChooser.setItemName(fileName);
            verChooser.execute();

            context.setCheckout("Check Out");
            JCaCheckout checkout = context.getCheckout();
            checkout.setCheckoutMode(JCaConst.CO_MODE_BROWSE);
            checkout.setPathOption(JCaConst.CO_OPTION_ALL_ITEM_TO_SAME_DIRECTORY);
            checkout.setClientDir(tempClientPath);
            checkout.setViewPath(viewPath);
            checkout.setReplaceFile(true);
            int iResult = checkout.execute();
            if (iResult != 0)
                throw new VcsException("CAHarvestProvider.checkoutTempFail:" + fileName);
        } catch (JCaHarvestException e) {
            throw new VcsException(e.getMessage());
        }
        File tempSigFile = new File(tempClientPath, "harvest.sig");
        tempSigFile.deleteOnExit();
        File tempFile = new File(tempClientPath, fileName);
        tempFile.deleteOnExit();
        try {
            return new FileInputStream(tempFile);
        } catch (FileNotFoundException _ex) {
            throw new VcsException("unknownError");
        }
    }

    public void addFile(String folderPath, String fileName, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: addFile(folderPath='" + folderPath + "' name='" + fileName + "')");
        }

        MapItem configItem = myConfiguration.findItem(folderPath);
        String viewPath = getViewPath(configItem, folderPath);
        JCaContext context = HarvestManager.getInstance().getContext(myProject);
        context.setProject(configItem.PROJECT);
        context.setState(configItem.STATE);
        context.setPackage(configItem.PACKAGE);
        if (context.getPackageId() == -1) throw new VcsException("No package.");
        String comment = (String) parameters;
        context.setCheckin("Check In Changes");
        JCaCheckin checkin = context.getCheckin();
        if (comment != null)
            checkin.setDescription(comment);
        checkin.setCheckinMode(JCaConst.HAR_UPDATE_AND_RELEASE);
        checkin.setClientDir(folderPath);
        //String sViewPath = context.getViewPath();
        checkin.setViewPath(viewPath);
        checkin.setDeleteAfterCI(false);
        checkin.setPathOption(JCaConst.CI_OPTION_PRESERVE_AND_CREATE_PATH);
        checkin.setItemOption(JCaConst.CI_FILTER_NEW_ITEMS_ONLY);
        String fileNames[] = new String[]{fileName};

        if (fileNames.length > 0) {
            checkin.setFileList(fileNames);
            int iResult = checkin.execute();
            if (iResult != 0)
                throw new VcsException("checkin fail." + context.getCheckinName() + folderPath + SEP + fileName);
        }

    }

    public void removeFile(String path, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: removeFile(filePath='" + path + "')");
        }
    }

    public void removeDirectory(String path, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: removeDirectory(path='" + path + "')");
        }
    }

    public void addDirectory(String parentPath, String name, Object parameters) throws VcsException {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: addFile(folderPath='" + parentPath + "' name='" + name + "')");
        }

        MapItem configItem = myConfiguration.findItem(parentPath);
        String viewPath = getViewPath(configItem, parentPath);
        JCaContext context = HarvestManager.getInstance().getContext(myProject);
        context.setProject(configItem.PROJECT);
        context.setState(configItem.STATE);
        context.setPackage(configItem.PACKAGE);
        if (context.getPackageId() == -1) throw new VcsException("No package.");
        String comment = (String) parameters;
        context.setCheckin("Check In Changes");
        JCaCheckin checkin = context.getCheckin();
        if (comment != null)
            checkin.setDescription(comment);
        checkin.setCheckinMode(JCaConst.HAR_UPDATE_AND_RELEASE);
        checkin.setClientDir(parentPath);
        //String sViewPath = context.getViewPath();
        checkin.setViewPath(viewPath);
        checkin.setDeleteAfterCI(false);
        checkin.setPathOption(JCaConst.CI_OPTION_PRESERVE_AND_CREATE_PATH);
        checkin.setItemOption(JCaConst.CI_FILTER_NEW_OR_EXISTING_ITEMS);
        String[] fileNames = listFile(parentPath, name).toArray(new String[0]);
        for (int i = 0; i < fileNames.length; i++) {
            String fileName = fileNames[i];
            fileNames[i] = fileName.substring(parentPath.length() + 1);
        }
        if (fileNames.length > 0) {
            checkin.setFileList(fileNames);
            int iResult = checkin.execute();
            if (iResult != 0)
                throw new VcsException("checkin fail.[iResult=" + iResult + "]:" + context.getCheckinName() + parentPath + SEP + name);
        }
    }

    private ArrayList<String> listFile(String parentPath, String name) {
        File[] files;
        File file = new File(parentPath + SEP + name);
        ArrayList<String> array = new ArrayList<String>();
        if (file.isDirectory()) {
            files = file.listFiles();
            for (File file1 : files) {
                if (file1.isDirectory()) {
                    array.addAll(listFile(file1.getParent(), file1.getName()));
                } else {
                    array.add(file1.getPath());
                }
            }
        } else {
            array.add(file.getPath());
        }
        return array;
    }

    public String getComponentName() {
        return "HarvestVcs";
    }

    public Configurable getConfigurable() {
        return new HarvestConfigurable(myProject);
    }

    public FileStatusProvider getFileStatusProvider() {
        return LocalVcsServices.getInstance(myProject).getFileStatusProvider();
    }

    public UpToDateRevisionProvider getUpToDateRevisionProvider() {
        return LocalVcsServices.getInstance(myProject).getUpToDateRevisionProvider();
    }

    public void doActivateActions() {
        LocalVcs.getInstance(myProject).markSourcesAsCurrent("Mark Sources as Current: Harvest Initialization");
    }

    public boolean markExternalChangesAsUpToDate() {
        return true;
    }

    public boolean supportsMarkSourcesAsCurrent() {
        return true;
    }

    public String getMenuItemText() {
        return "H&arvest";
    }

    public static AbstractVcs getInstance(com.intellij.openapi.project.Project project) {
        return project.getComponent(HarvestVcsAdapter.class);
    }

    public FileViewEnvironment getFileViewEnvironment() {
        return myFileViewEnvironment;
    }

    public CheckinEnvironment getCheckinEnvironment() {
        return myCheckinEnvironment;
    }

    public StandardOperationsProvider getStandardOperationsProvider() {
        return this;
    }

    public void checkinFile(String path, Object parameters, Map userData) throws VcsException {
        checkinFile(path, parameters);
    }

    public void addFile(String folderPath, String name, Object parameters, Map userData) throws VcsException {
        addFile(folderPath, name, parameters);
    }

    public void removeFile(String path, Object parameters, Map userData) throws VcsException {
        removeFile(path, parameters);
    }

    public void addDirectory(String parentPath, String name, Object parameters, Map userData) throws VcsException {
        addDirectory(parentPath, name, parameters);
    }

    public void removeDirectory(String path, Object parameters, Map userData) throws VcsException {
        removeDirectory(path, parameters);
    }


}