/**
 * Copyright (c) 2006, Van
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the VAN nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
package van.harvest.actions;

import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.fileEditor.FileDocumentManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vcs.*;
import com.intellij.openapi.vfs.VirtualFile;
import van.harvest.HarvestVcs;
import van.harvest.HarvestVcsAdapter;

import java.util.Arrays;
import java.util.List;

/**
 * @author mike
 */
public abstract class BasicAction extends AnAction {
    private static final Logger LOG = Logger.getInstance("#com.intellij.openapi.vcs.actions.VcsAction");

    public void actionPerformed(AnActionEvent event) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("enter: actionPerformed(id='" + ActionManager.getInstance().getId(this) + "')");
        }
        final DataContext dataContext = event.getDataContext();
        final Project project = (Project) dataContext.getData(DataConstants.PROJECT);

        final VirtualFile[] files = (VirtualFile[]) dataContext.getData(DataConstants.VIRTUAL_FILE_ARRAY);
        if (LOG.isDebugEnabled()) {
            LOG.debug("files='" + Arrays.asList(files) + "'");
        }
        if (files == null || files.length == 0) return;

        final AbstractVcs vcs = HarvestVcs.getInstance(project);
        if (!ProjectLevelVcsManager.getInstance(project).checkAllFilesAreUnder(vcs, files)) {
            FileDocumentManager.getInstance().saveAllDocuments();
        }

        final String actionName = getActionName(vcs);

        AbstractVcsHelper helper = AbstractVcsHelper.getInstance(project);
        com.intellij.openapi.localVcs.LvcsAction action = helper.startVcsAction(actionName);

        try {
            List<VcsException> exceptions = helper.runTransactionRunnable(vcs, new TransactionRunnable() {
                public void run(List exceptions) {
                    for (VirtualFile file : files) {
                        try {
                            execute(project, vcs, file, dataContext);
                        } catch (VcsException ex) {
                            ex.setVirtualFile(file);
                            exceptions.add(ex);
                        }
                    }
                }
            }, null);

            helper.showErrors(exceptions, actionName != null ? actionName : vcs.getDisplayName());
        } finally {
            if (actionName != null) {
                helper.finishVcsAction(action);
            }
        }
    }

    public void update(AnActionEvent e) {
        super.update(e);

        Presentation presentation = e.getPresentation();
        final DataContext dataContext = e.getDataContext();

        Project project = (Project) dataContext.getData(DataConstants.PROJECT);
        if (project == null) {
            presentation.setEnabled(false);
            presentation.setVisible(false);
            return;
        }

        VcsManager manager = VcsManager.getInstance(project);
        if (manager == null) {
            presentation.setEnabled(false);
            presentation.setVisible(false);
            return;
        }

        final AbstractVcs activeVcs = manager.getActiveVcs();
        if (activeVcs == null || !(activeVcs instanceof HarvestVcsAdapter)) {
            presentation.setEnabled(false);
            presentation.setVisible(false);
            return;
        }

        VirtualFile[] files = (VirtualFile[]) dataContext.getData(DataConstants.VIRTUAL_FILE_ARRAY);
        if (files == null || files.length == 0) {
            presentation.setEnabled(false);
            presentation.setVisible(true);
            return;
        }

        boolean enabled = true;

        for (VirtualFile file : files) {
            if (!isEnabled(project, activeVcs, file)) {
                enabled = false;
            }
        }

        presentation.setEnabled(enabled);
        presentation.setVisible(enabled);
    }


    private void execute(Project project, final AbstractVcs activeVcs, final VirtualFile file, DataContext context) throws VcsException {
        if (file.isDirectory()) {
            perform(project, ((HarvestVcsAdapter) activeVcs).getHarvestVcs(), file, context);
            ApplicationManager.getApplication().runWriteAction(new Runnable() {
                public void run() {
                    file.refresh(false, true);
                }
            });

            FileStatusManager.getInstance(project).fileStatusChanged(file);

            final VirtualFile[] children = file.getChildren();
            for (VirtualFile child : children) {
                execute(project, activeVcs, child, context);
            }
        } else {
            perform(project, ((HarvestVcsAdapter) activeVcs).getHarvestVcs(), file, context);
            ApplicationManager.getApplication().runWriteAction(new Runnable() {
                public void run() {
                    file.refresh(false, true);
                }
            });
            FileStatusManager.getInstance(project).fileStatusChanged(file);
        }
    }

    protected abstract String getActionName(AbstractVcs vcs);

    protected abstract boolean isEnabled(Project project, AbstractVcs vcs, VirtualFile file);

    protected abstract void perform(Project project, final HarvestVcs activeVcs, VirtualFile file, DataContext context) throws VcsException;
}
