/**
 * Copyright (c) 2006, Van
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the VAN nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
package van.harvest.actions;

import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.diff.*;
import com.intellij.openapi.fileTypes.FileType;
import com.intellij.openapi.fileTypes.FileTypeManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vcs.*;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.fileEditor.FileDocumentManager;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.ui.Messages;
import van.harvest.HarvestVcs;

import java.io.IOException;
import java.util.Arrays;

import sun.io.Converters;

/**
 * @author mike
 */
public class ShowDiffAction extends BasicAction {
    private static final Logger LOG = Logger.getInstance("#van.harvest.actions.ShowDiffAction");

    private byte[] getContentOf(VirtualFile file) throws IOException {
        Document document = FileDocumentManager.getInstance().getCachedDocument(file);
        if (document == null)
            return file.contentsToByteArray();
        return document.getText().getBytes(file.getCharset().name());
    }

    protected void perform(Project project, HarvestVcs activeVcs, VirtualFile file, DataContext context) throws VcsException {

        try {
            byte[] localContent = getContentOf(file);
            String upToDateFilePath = AbstractVcsHelper.getInstance(project).getUpToDateFilePath(file);
            if (upToDateFilePath == null) {
                upToDateFilePath = file.getPresentableUrl();
            }
            final byte[] vcsContent = activeVcs.getFileContent(upToDateFilePath, null);
            if (vcsContent == null) return;

            final Object modalContext = context.getData(DataConstants.IS_MODAL_CONTEXT);
            Object modalHint = modalContext != null && modalContext.equals(Boolean.TRUE) ?
                    DiffTool.HINT_SHOW_MODAL_DIALOG :
                    DiffTool.HINT_SHOW_FRAME;
            FileType fileType = FileTypeManager.getInstance().getFileTypeByFile(file);

            if (fileType.isBinary()) {
                if (Arrays.equals(localContent, vcsContent)) {
                    Messages.showMessageDialog("Binary contents are equal", "Contents Are Equal", Messages.getInformationIcon());
                } else {
                    Messages.showMessageDialog("Binary contents are different", "Contents Are Different", Messages.getInformationIcon());
                }
                return;
            }

            BinaryContent content = new BinaryContent(vcsContent, Converters.getDefaultEncodingName(), fileType);
            SimpleDiffRequest diffRequest = new SimpleDiffRequest(project, file.getPresentableUrl() + " diff");
            diffRequest.setContentTitles(activeVcs.getDisplayName() + " version", "LocalVersion");
            diffRequest.setContents(content, new FileContent(project, file));
            diffRequest.addHint(modalHint);
            DiffTool diffTool = DiffManager.getInstance().getDiffTool();
            if (diffTool.canShow(diffRequest)) diffTool.show(diffRequest);

        } catch (IOException e) {
            LOG.error(e);
        }
    }

    protected boolean isEnabled(Project project, AbstractVcs vcs, VirtualFile file) {
        return !file.isDirectory() && FileStatusManager.getInstance(project).getStatus(file) != FileStatus.ADDED;
    }

    protected String getActionName(AbstractVcs vcs) {
        return null;
    }
}
